// Copyright Marcus Del Favero 2024
// Licensed under the GNU AGPLv3 with an exception, see `README.md` for details
#[cfg(feature = "client")] mod renderer;
#[cfg(feature = "client")] mod mesh;
mod range_iter;
pub mod compressed;

#[cfg(feature = "client")] pub use renderer::Renderer as BlocksRenderer;
pub use range_iter::RangeIterator;

use std::num::NonZeroU32;

use glam::{UVec2, Vec2};

pub struct Blocks {
	block_size: NonZeroU32,
	grid_size: UVec2,
	data: Box<[bool]>,
}

pub const MAX_BLOCKS: usize = 1048576;

impl Blocks {
	pub fn new(block_size: NonZeroU32, grid_size: UVec2, data: Box<[bool]>) -> Blocks {
		Blocks { block_size, grid_size, data }
	}

	/**
	 * Returns an iterator over all blocks colliding with the square centred at
	 * `pos` with radius (half side length) of `radius`.
	 */
	pub fn iter_range(&self, pos: Vec2, radius: f32) -> RangeIterator {
		debug_assert!(radius > 0.0);
		RangeIterator::new(self, pos - Vec2::splat(radius), pos + Vec2::splat(radius))
	}
}
