// Copyright Marcus Del Favero 2024
// Licensed under the GNU AGPLv3 with an exception, see `README.md` for details
use serde::{Serialize, Deserialize};

use crate::player::update::{PlayerUpdate, PlayerBulkUpdate};
use crate::world::{World, update::WorldUpdate};
use crate::scores::Scores;
use crate::blocks::compressed::CompressedBlocks;

#[derive(Clone)]
pub enum Message {
	World(WorldMessage),
	Score(ScoreMessage),
	PlayerUpdate(PlayerUpdate, u64), // Client → Server, frequently sent updates of the player's input with an associated update count (to compress multiple messages)
	PlayerBulkUpdate(PlayerBulkUpdate), // Client → Server, optionally sent out as redundant player update information
}

#[derive(Clone, Serialize, Deserialize)]
pub enum WorldMessage {
	InitWorld { // Server → Client, when the client joins a game
		player_id: u32, // ID of the player to be joined
		world: World,
		blocks: CompressedBlocks,
	},
	WorldUpdate(WorldUpdate, u64), // Server → Client, sent for every update to the world's physics
	SanityCheck(World), // Server → Client, used for the client to check if the server's world matches theirs
}

#[derive(Clone, Serialize, Deserialize)]
pub enum ScoreMessage {
	InitScores(Scores), // Server → Client, the initial scores of the game
	Score(u32, Option<i32>), // Server → Client, sets score of player associated with the key to the value, removing that score if None
}
