// Copyright Marcus Del Favero 2024
// Licensed under the GNU AGPLv3 with an exception, see `README.md` for details
use glam::Vec2;
use strum::EnumCount;
use strum_macros::EnumCount as EnumCountMacro;

use super::{update::PlayerUpdate, input_state::InputState, direction::Direction};

use crate::game::config::action::{Action, ActionEvent};

#[derive(EnumCountMacro)]
pub(super) enum Movement { Left, Right, Down, Up }

#[derive(Default)]
pub struct Controller {
	movement_states: [bool; Movement::COUNT],
	shooting: bool,
	dir: Direction,
}

impl Controller {
	pub fn new() -> Controller { Controller::default() }

	pub fn action(&mut self, action: ActionEvent) {
		match action {
			ActionEvent::Action(action, state) => {
				let pressed = state.pressed();
				match action {
					Action::MoveLeft => self.movement_states[Movement::Left as usize] = pressed,
					Action::MoveRight => self.movement_states[Movement::Right as usize] = pressed,
					Action::MoveUp => self.movement_states[Movement::Up as usize] = pressed,
					Action::MoveDown => self.movement_states[Movement::Down as usize] = pressed,
					Action::Shoot => self.shooting = pressed,
					_ => (),
				}
			},
			ActionEvent::AllReleased => self.reset_action_state(),
		}
	}

	pub fn reset_action_state(&mut self) {
		self.movement_states = Default::default();
		self.shooting = false;
	}

	pub fn mouse_move_event(&mut self, cursor_pos: Vec2) {
		self.dir = Direction::from(cursor_pos);
	}

	pub fn get_update(&self) -> PlayerUpdate {
		PlayerUpdate { dir: self.dir, input_state: InputState::from_parts(self.movement_states, self.shooting) }
	}
}
